/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.talk

import com.kakao.sdk.auth.network.rxKapiWithOAuth
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.network.RxOperations
import com.kakao.sdk.partner.talk.model.*
import com.kakao.sdk.talk.RxTalkApiClient
import com.kakao.sdk.talk.model.FriendOrder
import com.kakao.sdk.talk.model.Friends
import com.kakao.sdk.talk.model.Order
import com.kakao.sdk.template.model.DefaultTemplate
import io.reactivex.Single

/**
 * @suppress
 */
val RxTalkApiClient.partnerApi: RxPartnerTalkApi
    get() = ApiFactory.rxKapiWithOAuth.create(RxPartnerTalkApi::class.java)

/**
 * 카카오 플랫폼 내 친구 목록 가져오기.
 *
 * @param friendType [FriendType]
 * @param friendFilter [FriendFilter]
 * @param friendOrder [FriendOrder]
 */
@JvmOverloads
fun RxTalkApiClient.friendsForPartner(
        friendType: FriendType? = null,
        friendFilter: FriendFilter? = null,
        friendOrder: FriendOrder? = null,
        offset: Int? = null,
        limit: Int? = null,
        order: Order? = null,
        countryCodes : List<String>? = null
): Single<Friends<PartnerFriend>> =
    partnerApi.friends(friendType, friendFilter, friendOrder, offset, limit, order, countryCodes?.joinToString(","))
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 카카오톡 채팅방 목록 가져오기.
 */
@JvmOverloads
fun RxTalkApiClient.chatList(
    filters: List<ChatFilter>? = null,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null
): Single<Chats> =
    partnerApi.chatList(filters?.joinToString(",") { it.value }, offset, limit, order)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 카카오톡 채팅방에 속한 멤버들 가져오기.
 *
 * 채팅방 아이디를 기준으로 조회하며 친구인 멤버만 조회할지 선택 가능.
 */
@JvmOverloads
fun RxTalkApiClient.chatMembers(
    chatId: Long,
    friendsOnly: Boolean? = null,
    includeProfile: Boolean? = null,
    token: Long? = null
): Single<ChatMembers> =
    partnerApi.chatMembers(chatId, friendsOnly, includeProfile, token)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 카카오 디벨로퍼스에서 생성한 서비스만의 커스텀 메시지 템플릿을 사용하여, 조회한 친구를 대상으로 카카오톡으로 메시지 전송.
 *
 * 템플릿을 생성하는 방법은 [메시지 템플릿 가이드](https://developers.kakao.com/docs/latest/ko/message/message-template) 참고.
 */
@JvmOverloads
fun RxTalkApiClient.sendCustomMessageForPartner(
    receiverUuids: List<String>,
    templateId: Long,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), templateId, templateArgs)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 카카오 디벨로퍼스에서 생성한 서비스만의 커스텀 메시지 템플릿을 사용하여, 특정 채팅방에 카카오톡으로 메시지 전송.
 *
 * 템플릿을 생성하는 방법은 [메시지 템플릿 가이드](https://developers.kakao.com/docs/latest/ko/message/message-template) 참고.
 */
@JvmOverloads
fun RxTalkApiClient.sendCustomMessageToChats(
    receiverChatIds: List<Long>,
    templateId: Long,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds), templateId, templateArgs)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 기본 템플릿을 사용하여, 조회한 친구를 대상으로 카카오톡 메시지 전송.
 */
fun RxTalkApiClient.sendDefaultMessageForPartner(
    receiverUuids: List<String>,
    template: DefaultTemplate
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), template)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 기본 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지 전송.
 */
fun RxTalkApiClient.sendDefaultMessageToChats(
    receiverChatIds: List<Long>,
    template: DefaultTemplate
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds), template)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 지정된 URL을 스크랩하여, 조회한 친구를 대상으로 카카오톡으로 메시지 전송.
 *
 * 스크랩 커스텀 템플릿 가이드를 참고하여 템플릿을 직접 만들고 스크랩 메시지 전송에 이용 가능.
 *
 * @see RxTalkApiClient.friendsForPartner
 */
@JvmOverloads
fun RxTalkApiClient.sendScrapMessageForPartner(
    receiverUuids: List<String>,
    requestUrl: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), requestUrl, templateId, templateArgs)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())

/**
 * 지정된 URL 을 스크랩하여, 특정 채팅방에 카카오톡 메시지 전송.
 *
 * 스크랩 커스텀 템플릿 가이드를 참고하여 템플릿을 직접 만들고 스크랩 메시지 전송에 이용 가능.
 */
@JvmOverloads
fun RxTalkApiClient.sendScrapMessageToChats(
    receiverChatIds: List<Long>,
    requestUrl: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds.map { it.toString() }), requestUrl, templateId, templateArgs)
        .compose(RxOperations.handleApiError())
        .compose(authOperations.handleApiError())
